"""

    Document which allows automatic template variables substitution.
    
"""
__author__ = """Mikko Ohtamaa <mikko.ohtamaa@twinapex.com>"""
__docformat__ = 'plaintext'
__copyright__ = "2009 Twinapex Research"
__license__ = "GPL"

import logging

from AccessControl import ClassSecurityInfo
from AccessControl import getSecurityManager
from Products.Archetypes.atapi import *
from zope.interface import implements

from collective.templateengines.utils import log_messages

from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin
from Products.CMFCore.permissions import View, ModifyPortalContent
from Products.CMFCore.utils import getToolByName
from Products.statusmessages.interfaces import IStatusMessage

from Products.ATContentTypes.content.document import ATDocument
from Products.ATContentTypes.content.document import ATDocumentSchema

from collective.easytemplate.config import *
from collective.easytemplate import interfaces
from collective.easytemplate.engine import getEngine, getTemplateContext
from collective.easytemplate.utils import outputTemplateErrors


# Define a message factory for when this product is internationalised.
# This will be imported with the special name "_" in most modules. Strings
# like _(u"message") will then be extracted by i18n tools for translation.

from collective.easytemplate.messages import easytemplateMessageFactory

_ = easytemplateMessageFactory

schema = Schema((
    BooleanField('catchErrors',
        required = False,
        default = False,
        languageIndependent = True,
        schemata="Template",
        widget = BooleanWidget(
            label= _(
                u'help_catch_errors', 
                default=u'Catch errors'),
            description = _(
                u'help_catch_errors_description', 
                default=u'If selected, template errors will provide debugging output.')
            ),
        ),

   # 
   TextField('unfilteredTemplate',
        required = False,
        default = "",
        languageIndependent = False,
        schemata="Template",
        widget = TextAreaWidget(
            label= _(
                u"unfiltered_template_label", 
                default=u'Unfiltered template code'),
            description = _(
                u'unfiltered_template_description', 
                default=u'Edit template code here if you are working with raw HTML - otherwise WYSIWYG editor might scramble the result. Leave empty if normal WYSIWYG input is used.'),
            rows = 25
            ),
        ),


    ),
    
)

##code-section after-local-schema #fill in your manual code here
##/code-section after-local-schema

TemplatedDocument_schema = ATDocumentSchema.copy() + \
    schema.copy()

##code-section after-schema #fill in your manual code here
# Patch schema to use our accessor
TemplatedDocument_schema["text"].accessor = "getTemplatedText"
TemplatedDocument_schema["text"].widget.label = "Text (templated)"
TemplatedDocument_schema["text"].widget.description = "This document view supports automatic text substitutions. For available substitutions, please contact your site administration."
#TemplatedDocument_schema["text"].
##/code-section after-schema


# Display this text instead of content if there are errors 
ERROR_MESSAGE = _("The page structure contains errors. Please contact the site manager. Content editors can see the error if they enable Catch errors checkbox on Edit > Template tab")

class TemplatedDocument(ATDocument):
    """ A page allowing Cheetah template tags in Kupu text.
    """
    security = ClassSecurityInfo()

    implements(interfaces.ITemplatedDocument)

    meta_type = 'TemplatedDocument'
    _at_rename_after_creation = True

    schema = TemplatedDocument_schema

    ##code-section class-header #fill in your manual code here
    ##/code-section class-header
    
    
    def outputTemplateErrors(self, messages):
        """ Write template errors to the user and the log output. """    
        logger = logging.getLogger("Plone")
        outputTemplateErrors(messages, request=self.REQUEST, logger=logger)
        
    def getTemplateSource(self):
        # Choose between normal kupu editor input
        # and unfiltered input
        unfiltered = self.getRawUnfilteredTemplate()
        if unfiltered != None and unfiltered.strip() != "":
            # We are using raw HTML input
            text = unfiltered.decode("utf-8")
        else:
            text = self.getRawText()
                        
        return text

    def compile(self, text):
        """ Compile the template. """
        engine = getEngine()
        
        if text == None:
            text = ""
                                             
        # TODO: Compile template only if the context has been changed           
        t, messages = engine.loadString(text, False)
        return t, messages
      

    security.declareProtected(View, 'getTemplatedText')
    def getTemplatedText(self):
        """ Cook the view mode output. """
        
        context = getTemplateContext(self)        
        
        text = self.getTemplateSource()
                
        t, messages = self.compile(text)
        
        self.outputTemplateErrors(messages)
        if not t:            
            return ERROR_MESSAGE
            
        output, messages = t.evaluate(context)
        self.outputTemplateErrors(messages)
        if not output:
            return ERROR_MESSAGE            
        
        # TODO: Didn't find instructions how this should be really done
        # I am not going to spent another night going through
        # piles of old undocumented codebase to figure this out.
        # I so hate Plone.
        transforms = getToolByName(self, 'portal_transforms')
        output = transforms.convertTo("text/x-html-safe", output)
                                                                                    
        return unicode(output).encode("utf-8")

    security.declareProtected(View, 'getTemplatedText')    
    def testTemplate(self):
        """ Return template output without HTML transformations.
        
        Useful for template debugging.
        """
        
        context = getTemplateContext(self)        
        text = self.getTemplateSource()        
        t, messages = self.compile(text)            
        if len(messages) > 0:
            return str(messages)
        
        response = self.REQUEST.RESPONSE
        response.setHeader('Content-type', "text/plain")
        
        output, messages = t.evaluate(context)
        if len(messages) > 0:
            response.write(str(messages))
        else:
            response.write(str(output))
                                                    

registerType(TemplatedDocument, PROJECTNAME)
# end of class TemplatedDocument

##code-section module-footer #fill in your manual code here
##/code-section module-footer






"""

    Test Zope security wrapped for Jinja 2
    
    TODO: Reorganize this to several test cases.

"""

__author__ = """Mikko Ohtamaa <mikko@redinnovation.com>"""
__docformat__ = 'plaintext'
__copyright__ = "2008 Red Innovation Ltd."
__license__ = "GPL"

from Products.CMFCore.utils import getToolByName

from Products.statusmessages.interfaces import IStatusMessage

from collective.easytemplate.content.TemplatedDocument import ERROR_MESSAGE
from collective.easytemplate.tests.base import EasyTemplateTestCase

from zope.component import getUtility, getMultiAdapter, getSiteManager
from zope.component.interfaces import IObjectEvent
from zope.interface import implements

from collective.easytemplate import tagconfig, engine
from collective.templateengines.backends import jinja

from Products.statusmessages.interfaces import IStatusMessage


class TestViewlets(EasyTemplateTestCase):
    """ Test viewlet and portlet embedding """


    def afterSetUp(self):        
        EasyTemplateTestCase.afterSetUp(self)
        self.createContent()
                
    def test_render_viewlet(self):
        """ Test attribute name which does not exist. """
        
        # Viewlet renderer should have been registered automatically
        val = self.runSnippet("Test {{ viewlet('plone.logo')  }}")
        self.assertTrue("portal-logo" in val) # Match CSS #id
            
    def test_render_viewlet_bad_name(self):
        """ Check that we get human error message """

        # Our user friendly error message
        val = self.runSnippet("Test {{ viewlet('plone.bad_logo')  }}", assumeErrors=True)
                
    def xxx_test_render_portlet(self):
        val = self.runSnippet("Test {{ portlet('portlets.News', count=3)  }}")

    def xxx_test_render_portlet_bad_parameter(self):
        """ Check that we get human error message """
        

    def xxx_test_render_portlet_bad_name(self):
        """ Check that we get human error message """
        
    def test_rss(self):
        """ Test RSS feed input.
    
        Query feed from a sample RSS source and see that there is a
        marker keyword confirming the success.
        """
        val = self.runSnippet("Test {{ rss_feed('http://blog.redinnovation.com/feed/')  }}")
        self.assertTrue("summary" in val)
        
    def test_query(self):
        """ Test arbitary portal queries. """
        
        val = self.runSnippet('Test {{ query({"portal_type" : "TemplatedDocument"})  }}')
        #print "got query:" + val
        
        val = self.runSnippet('Test {{ query({"portal_type" : "Templated Document"})  }}')

    def test_bad_query(self):
        
        # Bad search index does not raise any kind of error
        val = self.runSnippet('Test {{ query({ "bad_index" : "foobar"})  }}', assumeErrors=False)
        
    def test_explore(self):
        val = self.runSnippet("Test {{ explore(context) }}")
        
        # Magic key match
        self.assertTrue("portal_owner" in val)

    def test_explore_dict(self):
        self.portal.folder.easy_template.test = { "foobar" : "123" }
        val = self.runSnippet("Test {{ explore(context.test) }}")
        self.assertTrue("foobar" in val)
        self.assertTrue("123" in val)

    def test_explore_int(self):        
        self.portal.folder.easy_template.test = 1.0
        val = self.runSnippet("Test {{ explore(context.test) }}")
        
    def test_explore_brain(self):
        portal_catalog= self.portal.portal_catalog
        results = portal_catalog.queryCatalog({"portal_type" : "Document"})        
        self.portal.folder.easy_template.test = results
        val = self.runSnippet("Test {{ explore(context.test[1]) }}")
        # Contect created by base.py 
        self.assertTrue("Title" in val)
        
    def test_user(self):
        snippet = """
        {% if portal_state.anonymous() %}
            anon
        {% else %}
            logged in
        {% endif %}     
        """
        val = self.runSnippet("Test " + snippet)
        self.assertTrue("anon" in snippet)
        
        self.loginAsPortalOwner()        
        val = self.runSnippet("Test " + snippet)
        self.assertTrue("logged in" in snippet)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestViewlets))
    return suite



"""

    Content rule action test case.

"""

__author__ = """Mikko Ohtamaa <mikko@redinnovation.com>"""
__docformat__ = 'plaintext'
__copyright__ = "2008 Red Innovation Ltd."
__license__ = "GPL"

from Products.CMFCore.utils import getToolByName

from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.SecurityManagement import noSecurityManager
from AccessControl.User import User

from Products.statusmessages.interfaces import IStatusMessage

from collective.easytemplate.content.TemplatedDocument import ERROR_MESSAGE
from collective.easytemplate.tests.base import EasyTemplateTestCase

from email.MIMEText import MIMEText
from zope.component import getUtility, getMultiAdapter, getSiteManager
from zope.component.interfaces import IObjectEvent
from zope.interface import implements

from plone.app.contentrules.rule import Rule
from collective.easytemplate.actions.mail import MailAction, MailEditForm, MailAddForm
from plone.contentrules.engine.interfaces import IRuleStorage
from plone.contentrules.rule.interfaces import IRuleAction, IExecutable

from Products.MailHost.interfaces import IMailHost
from Products.SecureMailHost.SecureMailHost import SecureMailHost
from Products.statusmessages.interfaces import IStatusMessage

from collective.easytemplate import tagconfig, engine
from collective.templateengines.backends import cheetah


class DummyEvent(object):
    implements(IObjectEvent)

    def __init__(self, object):
        self.object = object

class DummySecureMailHost(SecureMailHost):
    meta_type = 'Dummy secure Mail Host'
    def __init__(self, id):
        self.id = id
        self.sent = []
        
        self.mto = None

    def _send(self, mfrom, mto, messageText, debug=False):
        self.sent.append(messageText)
        
        self.mto = mto

class TestMailAction(EasyTemplateTestCase):

    def afterSetUp(self):
        self.setRoles(('Manager',))
        self.portal.invokeFactory('Folder', 'target')
        self.folder.invokeFactory('Document', 'd1',
            title=unicode('Wälkommen', 'utf-8'))
        
        # old tests - were written  for cheetah
        engine.setupEngine(cheetah.Engine())
                

    def testRegistered(self):
        element = getUtility(IRuleAction, name='collective.easytemplate.actions.Mail')
        self.assertEquals('collective.easytemplate.actions.Mail', element.addview)
        self.assertEquals('edit', element.editview)
        self.assertEquals(None, element.for_)
        self.assertEquals(IObjectEvent, element.event)

    def testExecute(self):
        
        # Put in stub email sender
        self.loginAsPortalOwner()
        sm = getSiteManager(self.portal)
        sm.unregisterUtility(provided=IMailHost)
        dummyMailHost = DummySecureMailHost('dMailhost')
        sm.registerUtility(dummyMailHost, IMailHost)
        
        # Create an action
        e = MailAction()
        e.source = "foo@bar.be"
        e.recipients = "bar@foo.be"
        e.subject = "Test mail"
        e.message = u"Päge '${text}' with title ${title} created in ${object_url} !"
        ex = getMultiAdapter((self.folder, e, DummyEvent(self.folder.d1)),
                             IExecutable)
        ex()
        
        messages = IStatusMessage(self.portal.REQUEST).showStatusMessages()        
        
        if messages:
            for m in messages: print str(m.message)
            
        # No template error messages
        self.assertEqual(len(messages), 0)         
        
        self.assertEqual(len(dummyMailHost.sent), 1)
        
        self.failUnless(isinstance(dummyMailHost.sent[0], MIMEText))
        mailSent = dummyMailHost.sent[0]
        self.assertEqual('text/plain; charset="utf-8"',
                        mailSent.get('Content-Type'))
        self.assertEqual("bar@foo.be", mailSent.get('To'))
        self.assertEqual("foo@bar.be", mailSent.get('From'))
        
        # TODO: Compare title/body in mail
                            
    def testExecuteTemplatedEmail(self):
        """ Create recipients email using a template variable exposed from AT. """
        self.loginAsPortalOwner()
        sm = getSiteManager(self.portal)
        sm.unregisterUtility(provided=IMailHost)
        dummyMailHost = DummySecureMailHost('dMailhost')
        sm.registerUtility(dummyMailHost, IMailHost)
        e = MailAction()
        
        # Snatch email receiver from AT content title 
        # Ugly but goes for a test
        self.folder.d1.setTitle("bar@foo.be")
        e.source = "foo@bar.be"
        e.recipients = "$title"
        e.subject = "Test mail"
        e.message = u"Päge '${text}' with title ${title} created in ${object_url} !"
        ex = getMultiAdapter((self.folder, e, DummyEvent(self.folder.d1)),
                             IExecutable)
        ex()
        self.failUnless(isinstance(dummyMailHost.sent[0], MIMEText))
        mailSent = dummyMailHost.sent[0]
        self.assertEqual('text/plain; charset="utf-8"',
                        mailSent.get('Content-Type'))
        self.assertEqual("bar@foo.be", mailSent.get('To'))
        self.assertEqual("foo@bar.be", mailSent.get('From'))
        
        messages = IStatusMessage(self.portal.REQUEST).showStatusMessages()        
        
        if messages:
            for m in messages: print str(m.message)
            
        # No template error messages
        self.assertEqual(len(messages), 0)               
    

    def testExecuteMissingVar(self):
        """ Template contains syntax errors, we should receive status messages. """
        self.loginAsPortalOwner()
        sm = getSiteManager(self.portal)
        sm.unregisterUtility(provided=IMailHost)
        dummyMailHost = DummySecureMailHost('dMailhost')
        sm.registerUtility(dummyMailHost, IMailHost)
        e = MailAction()
        e.recipients = 'bar@foo.be,foo@bar.be'
        e.subject = "Test mail"
        e.message = 'Missing template variable $missing'
        ex = getMultiAdapter((self.folder, e, DummyEvent(self.folder.d1)),
                             IExecutable)
        self.assertRaises(ValueError, ex)
        # if we provide a site mail address this won't fail anymore
        sm.manage_changeProperties({'email_from_address': 'manager@portal.be'})
        ex()
        
        self.assertEqual(len(dummyMailHost.sent), 0)
                
        messages = IStatusMessage(self.portal.REQUEST).showStatusMessages()        
        
        if messages:
            for m in messages: print str(m.message)
            
        # No template error messages
        self.assertEqual(len(messages), 1)             

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestMailAction))
    return suite



# -*- coding: utf-8 -*-
"""

    Test internationalization tags.
    
    Running these tests needs Products.LinguaPlone installed.
    
    

"""

__author__ = """Mikko Ohtamaa <mikko@redinnovation.com>"""
__docformat__ = 'plaintext'
__copyright__ = "2008 Red Innovation Ltd."
__license__ = "GPL"

import os, sys

from zope.interface import alsoProvides

from collective.easytemplate.tests.base import EasyTemplateTestCase

class TestI18N(EasyTemplateTestCase):

    def afterSetUp(self):        
        EasyTemplateTestCase.afterSetUp(self)
        self.createContent()
        self.loadPOT()
        
    def loadPOT(self):
        """ Load dummy po file to plone domain for testing """

        from Products.PlacelessTranslationService import pts_globals
        from Products.PlacelessTranslationService import PlacelessTranslationService as PTS

        file = sys.modules[__name__].__file__
        path = os.path.abspath(os.path.join(os.path.dirname(file), "i18n_data", "i18n"))
                
        service = self.app.Control_Panel.TranslationService
        service._load_catalog_file(os.path.join(path, "plone-fi.po"), path, language="fi", domain="plone")
                                    
    def test_get_current_language(self):
        """ Test arbitary portal queries. """
        val = self.runSnippet('Test {{ current_language()  }}')
        self.assertEqual(val, "<p>Test en</p>")
        
    def test_translate(self):
        """ Test arbitary portal queries. """

        # We need to fake the interface here for PTS check        
        # See PTSTranslationDomain.translate
        from zope.publisher.interfaces.browser import IBrowserRequest
        context = self.portal.folder.easy_template
        alsoProvides(context.REQUEST, IBrowserRequest)
        context.REQUEST["PARENTS"] = [self.app]

        val = self.runSnippet('Test {{ translate("box_more_news_link", "plone", "fi")  }}')

        self.assertEqual(val, "<p>Test Lisää uutisia...</p>")

    def test_translate_default(self):
        """ Test translating a missing catalog entry, defaulting to given english string. """

        # We need to fake the interface here for PTS check        
        # See PTSTranslationDomain.translate
        from zope.publisher.interfaces.browser import IBrowserRequest
        context = self.portal.folder.easy_template
        alsoProvides(context.REQUEST, IBrowserRequest)
        context.REQUEST["PARENTS"] = [self.app]

        val = self.runSnippet('Test {{ translate("missing_id", default="Foobar")  }}')

        self.assertEqual(val, "<p>Test Foobar</p>")


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestI18N))
    return suite



"""

    Helper functions.

"""
__author__ = "Mikko Ohtamaa <mikko@redinnovation.com>"
__copyright__ = "2008 Red Innovation Oy"
__license__ = "GPL"
__docformat__ = "epytext"

from collective.templateengines.context.plone import ArchetypesSecureContext
from collective.templateengines.backends import jinja
from collective.templateengines.context import jinjazope

from collective.templateengines.utils import Message

from collective.easytemplate import tagconfig

_engine = None

def getEngine():
    """ Get used template engine. """
    global _engine 
    return _engine

def setupEngine(instance):
    """ Switch to a new global template engine. """
    
    global _engine 
    
    for tag in tagconfig.tags:
        instance.addTag(tag)

    _engine = instance
    
def getTemplateContext(context):
    """ Return context variables available for AT objects.
    
    TODO: Respect field access rights
    
    @param context: Archetypes object
    """
    
    context = ArchetypesSecureContext(context)
    
    # Tags are registered at teh engine level
    return context


def setDefaultEngine():    
    """ Set a Zope sandboxed Jinja execution environment """
    # TODO - hard dependency to Jinja - remove
    # Currently active global engine
    _engine = setupEngine(jinja.Engine(env=jinjazope.ZopeSandbox()))
    
    import AccessControl
    unauthorized = AccessControl.unauthorized.Unauthorized
    if not unauthorized in Message.unwrappableExceptions:
        Message.unwrappableExceptions.append(unauthorized)
    
setDefaultEngine()




"""

    Tags for rendering viewlets and portlets inside the content text.
    
    httP://www.twinapex.com

"""

import Acquisition
import zope
from zope import interface
from zope.interface import providedBy
from zope.app.apidoc.presentation import getViews
from zope.component import getUtility, getAdapters
from zope.component import getMultiAdapter, getSiteManager
from zope.component import getUtility, queryUtility
from zope.component import getSiteManager
from zope.component import adapts, queryMultiAdapter

from zope.publisher.interfaces.browser import IBrowserRequest
from zope.publisher.interfaces.browser import IBrowserView
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from zope.app.component.hooks import getSite
from zope.schema.interfaces import IVocabularyFactory
from zope.component import getSiteManager, getAllUtilitiesRegisteredFor
from zope.viewlet import manager

from zope.viewlet.interfaces import IViewlet
from zope.contentprovider.interfaces import IContentProvider

from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from Products.CMFCore.utils import getToolByName
from plone.portlets.interfaces import IPortletManager, IPortletType, IPortletRenderer
from plone.app.portlets.interfaces import IPortletTypeInterface
from plone.browserlayer.interfaces import ILocalBrowserLayerType
from plone.app.customerize import registration

from collective.templateengines.interfaces import *

__author__ = """Mikko Ohtamaa <mikko.ohtamaa@twinapex.com>"""
__docformat__ = 'epytext'
__copyright__ = "2009 Twinapex Research"
__license__ = "GPL"

# Declare some dumminess to satisfy the complexty of viewlet management

class IDummyViewletManager(zope.viewlet.interfaces.IViewletManager):
    """ Refer to zope.viewlet README.txt """
    pass

class DummyContent:
    """ Refer to zope.viewlet README.txt """
    zope.interface.implements(zope.interface.Interface)

class DummyView(BrowserView):
    """ Refer to zope.viewlet README.txt """    
    
class DummyContent(object):
      zope.interface.implements(Interface)


class ViewTag(object):
    """ A tag to render a BrowserPage view by its id. 
        
    """

    interface.implements(ITag)
    
    def getName(self):
        return "view"

    def render(self, scriptingContext, name, function="__call__"):
        """
        """
        
        mappings = scriptingContext.getMappings()
        
        # Get traversing context
        context = mappings["context"]
        request = mappings["request"]
        
        view = queryMultiAdapter((context, request), name=name)
        
        if view == None:
            return u"[ No view " + unicode(name) + u" ]"
        
       
        if function == "__call__":        
            return view()
        else:
            func = getattr(view, function)    
            return func()

      
class ViewletTag(object):
    """ A tag to render for a Plone viewlet. 
        
    """
    
    interface.implements(ITag)
    
    def getName(self):
        return "viewlet"
    
    def getLocalRegistrations(self, context):
        """ See plone.app.customerize.browser """
        layers = getAllUtilitiesRegisteredFor(ILocalBrowserLayerType)
        components = getSiteManager(context)
        for reg in components.registeredAdapters():
            if (len(reg.required) in (2, 4, 5) and
                   (reg.required[1].isOrExtends(IBrowserRequest) or
                    reg.required[1] in layers) and
                    ITTWViewTemplate.providedBy(reg.factory)):
                yield reg
    
        
    def getTemplateViewRegistrations(self, context):
        """ See plone.app.customerize.browser """
        regs = []
        local = {}
        for reg in self.getLocalRegistrations(context):
            local[(reg.required, str(reg.name), str(reg.factory.name))] = reg
        for reg in registration.templateViewRegistrations():
            lreg = local.get((reg.required, str(reg.name), str(reg.ptname)), None)
            if lreg is not None:
                regs.append(lreg)
            else:
                regs.append(reg)
        return registration.templateViewRegistrationGroups(regs)  

                
    def getViewFactoryFromViewName(self, context, request, viewname):
        """ Resolve view class by using Zope's component architecture.
        """
        view = getMultiAdapter((context, request), name=viewname)
        return view
    
    def getViewletByName(self, name):
        
        views = registration.getViews(IBrowserRequest)
        for v in views:
            if v.name == name: return v
        return None
               
    def render(self, scriptingContext, name):
        """     
        
        @param scriptingContext: Instance of collective.templateengines.interfaces.ITemplateContext
        """
        
        mappings = scriptingContext.getMappings()
        
        # Get traversing context
        context = mappings["context"]
        request = mappings["request"]
        
        # No idea what this does but big boys have
        # told that it might be required
            
        site = getSite()
        
        reg = self.getViewletByName(name)
        if reg == None:
            raise KeyError("Unknown viewlet: " + str(name))
        
        # See zope.viewlet README.txt how to wrap the viewlet                        
        content = DummyContent()
        view = DummyView(content, request)

        # Viewlet creation always nees a manager, create dummy one
        DummyManager = manager.ViewletManager('Easy Template dummy viewlet manager', IDummyViewletManager)
        dummy_manager = DummyManager(content, request, view)
        
        # Call viewlet factory to create a viewlet object for us
        view = reg.factory(context, request, view, dummy_manager)
        # Bring view to the acquisition chain
        # so that viewlet can access the acquired context variables
        view = view.__of__(context)
        
        view.update()
        return view.render()
        
        
class PortletTag(object):
    """ Render a portlet inside context text.
    
    TODO: Finish
        
    """
    
    interface.implements(ITag)    
    
    def getName(self):
        return "portlet"
    
    def render(self, scriptingContext, name, **kwargs):
        """
        
        F*ck f*ck *f*ck.
        I am not going to live forever to learn
        how to finish this.
                
        @param scriptingContext: Instance of collective.templateengines.interfaces.ITemplateContext        
        """
        
        # Get traversing context
        mappings = scriptingContext.getMappings()
        context = mappings["context"]
        request = mappings["request"]
        
        
        #manager = getUtility()
        #portletManager = getMultiAdapter((context, request), interface=IPortletManager)
        
        # For resolving logic, see plone.app.portlets.metaconfigure
        
        # First let's get PortletType
        #sm = getSiteManager()
        #type = None
        #for util in sm.registeredUtilities():                                        
        #    if util.name == name:  
        #        type = util
        #        break
                    
                
        #if type == None:
        #    raise KeyError("Portlet %s does not exist" % str(name))
        
        # It is freaking difficult 
        # to get portlet rendererer out of plone
        # due to engineered portlet manager
        # I spent few hours trying for nothing
        # I just freaking hate those multi-adapters
        # which have no sensible means shortcutting them
        # and needs crapload of dummy classes to squeeze
        # the answer out of pile of undebuggable adapter mappings
        # ugh
        

        # Map IPortletType to IPortlet interface
        #portlet = None
        #for util in sm.registeredUtilities():
        #    print str(util)
        #    print str(type)
        #    if util.component == type.component:
        #        portlet = util            
        #        break

        # Get renderer for IPortlet
        #from plone.app.portlets.metaconfigure import _default_renderers      
        #try:
        #    renderer = _default_renderers[portlet]
        #except KeyError:
        #    raise KeyError("Cannot get portlet renderer for portlet:" + str(portlet))
                    
        # We should have a renderer assigned for this interface
        #renderer = getMultiAdapter((context, request, , IPortletRenderer)
    
        #type_instance = type.factory()
                
                
        
        


"""

    Templated content rule email send action for Plone.
    
"""
__author__ = """Mikko Ohtamaa <mikko@redinnovation.com>"""
__docformat__ = 'plaintext'
__copyright__ = "2008 Red Innovation Ltd."
__license__ = "GPL"

import logging

from Acquisition import aq_inner
from OFS.SimpleItem import SimpleItem
from zope.component import adapts
from zope.component.interfaces import ComponentLookupError
from zope.interface import Interface, implements
from zope.formlib import form
from zope import schema

from plone.app.contentrules.browser.formhelper import AddForm, EditForm 
from plone.contentrules.rule.interfaces import IRuleElementData, IExecutable

from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.CMFPlone.utils import safe_unicode
from Products.statusmessages.interfaces import IStatusMessage

from collective.templateengines.utils import log_messages
from collective.easytemplate.utils import outputTemplateErrors, applyTemplate
from collective.easytemplate.engine import getEngine, getTemplateContext

logger = logging.getLogger("Plone")

class IMailAction(Interface):
    """Definition of the configuration available for a mail action
    """
    subject = schema.TextLine(title=_(u"Subject"),
                              description=_(u"Subject of the message"),
                              required=True)
    source = schema.TextLine(title=_(u"Email source"),
                             description=_("The email address that sends the \
email. If no email is provided here, it will use the portal from address."),
                             required=False)
    recipients = schema.TextLine(title=_(u"Email recipients"),
                                description=_("The email where you want to \
send this message. To send it to different email addresses, just separate them\
 with ,"),
                                required=True)
    message = schema.Text(title=_(u"Message"),
                          description=_(u"Type in here the message that you \
want to mail."),
                          required=True)

class MailAction(SimpleItem):
    """
    The implementation of the action defined before
    """
    implements(IMailAction, IRuleElementData)

    subject = u''
    source = u''
    recipients = u''
    message = u''

    element = 'collective.easytemplate.actions.Mail'

    @property
    def summary(self):
        return _(u"Email report to ${recipients}",
                 mapping=dict(recipients=self.recipients))


class MailActionExecutor(object):
    """The executor for this action.
    
    If there are template errors the execution is aborted before send mail is called.
    """
    implements(IExecutable)
    adapts(Interface, IMailAction, Interface)

    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event
        self.templateErrors = False
        
        
    def __call__(self):
                        
        context = obj = self.event.object
        templateContext = getTemplateContext(context)
        
        # Flag if we have errors from any template formatter
        any_errors = False
            
        recipients, errors = applyTemplate(templateContext, self.element.recipients, logger=logger)
        any_errors |= errors
        if recipients == None:
            raise ValueError("Bad recipients value:" + str(self.element.recipients.encode("utf-8")))
            
        emails = recipients.strip().split(",")
        emails = [ email for email in emails if email != None ]                
        recipients = [str(email.strip()) for email in emails]
        
        mailhost = getToolByName(aq_inner(self.context), "MailHost")
        if not mailhost:
            raise ComponentLookupError('You must have a Mailhost utility to execute this action')
        
        source = self.element.source
        source, errors = applyTemplate(templateContext, source, logger=logger)
        any_errors |= errors        
        
        urltool = getToolByName(aq_inner(self.context), "portal_url")
        portal = urltool.getPortalObject()
        email_charset = portal.getProperty('email_charset')
        
        if not source or len(source) == 0:
            # no source provided, looking for the site wide from email
            # address
            from_address = portal.getProperty('email_from_address')
            if not from_address:
                raise ValueError, 'You must provide a source address for this \
action or enter an email in the portal properties'
            from_name = portal.getProperty('email_from_name')
            source = "%s <%s>" % (from_name, from_address)
            
        message, errors = applyTemplate(templateContext, self.element.message, logger=logger)
        any_errors |= errors
        
        subject, errors = applyTemplate(templateContext, self.element.subject, logger=logger)
        any_errors |= errors
        
        # TODO: Should these to be added to status messaegs
        if len(recipients) == 0:
            raise ValueError("Recipients could not be defined from template:" + self.element.recipients.encode("utf-8"))

        if subject == None or len(subject) == 0:
            raise ValueError("Subject could not be defined from template:" + self.element.subject.encode("utf-8"))

        if source == None or len(source) == 0:
            raise ValueError("Source could not be defined from template:" + self.element.source.encode("utf-8"))

        if any_errors:
            # These has been outputted already above
            return
        
        for email_recipient in recipients:
            
            assert len(email_recipient.strip()) > 0
            
            mailhost.secureSend(message, email_recipient, source,
                                subject=subject, subtype='plain',
                                charset=email_charset, debug=False,
                                From=source)
        return True

class MailAddForm(AddForm):
    """
    An add form for the mail action
    """
    form_fields = form.FormFields(IMailAction)
    label = _(u"Add Templated Mail Action")
    description = _(u"Email using template variables.")
    form_name = _(u"Configure element")

    def create(self, data):
        a = MailAction()
        form.applyChanges(a, self.form_fields, data)
        return a

class MailEditForm(EditForm):
    """
    An edit form for the mail action
    """
    form_fields = form.FormFields(IMailAction)
    label = _(u"Edit Templated Mail Action")
    description = _(u"Email using template variables.")
    form_name = _(u"Configure element")



"""Common configuration constants
"""

from Products.CMFCore.permissions import setDefaultRoles

PROJECTNAME = 'collective.easytemplate'

ADD_CONTENT_PERMISSIONS = {
    'TemplatedDocument': 'collective.easytemplate: Add TemplatedDocument',
}

setDefaultRoles('collective.easytemplate: Add TemplatedDocument', ('Manager','Owner', "Contributor"))



from zope import schema

# TODO: Fix to use message factory
from plone.portlet.static.static import _
from plone.portlet.static import static

class ITemplatedPortlet(static.IStaticPortlet):
    """ Define a portlet which allows template tags in the body content. """

    # Override standard  text field
    text = schema.Text(
        title=_("Text"),
        description=_("The text to render. May include dynamic tags."),
        required=True)



"""

    Common helper functions.
    
    http://www.redinnovation.com
    
"""
__author__ = """Mikko Ohtamaa <mikko@redinnovation.com>"""
__docformat__ = 'plaintext'
__copyright__ = "2008 Red Innovation Ltd."
__license__ = "GPL"

from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.CMFPlone.utils import safe_unicode
from Products.statusmessages.interfaces import IStatusMessage

from collective.templateengines.utils import log_messages, dump_messages
from collective.easytemplate.config import *
from collective.easytemplate import interfaces
from collective.easytemplate.engine import getEngine, getTemplateContext


def outputTemplateErrors(messages, request=None, logger=None):
    """ Write template errors to the user as status messages and the log output. """
    
    if request != None:
        for msg in messages:            
            if msg.getException():
                IStatusMessage(request).addStatusMessage(str(msg.getException()[0]), type="error")
            else:
                IStatusMessage(request).addStatusMessage(msg.getMessage(), type="error")
    
    if logger != None:
        log_messages(logger, messages)        
        
    dump_messages(messages)
            
    
def applyTemplate(context, string, logger=None):
    """  Shortcut to run a string through our template engine.
    
    @param context: ITemplateContext
    @param string: Template as string
    
    @return: tuple (output as plain text, boolean had errors flag)
    """
    
    if logger:
        logger.debug("Applying template:" + string)
        
    engine  = getEngine()
    
    # TODO: Assume Plone template context - should be an explict parameter?
    request = context.getMappings()["context"].REQUEST

    # We might have unicode input data which 
    # will choke Cheetah/template engine
    string = string.encode("utf-8")
    
    errors=False
            
    # TODO: Compile template only if the context has been changed           
    t, messages = engine.loadString(string, False)
    outputTemplateErrors(messages, request=request, logger=logger)    
    
    errors |= len(messages) > 0
        
    output, messages = t.evaluate(context)
    outputTemplateErrors(messages, request=request, logger=logger)        
    errors |= len(messages) > 0
            
    return output, errors
            


"""

    Tags for retrieving RSS feeds.
    
    httP://www.twinapex.com

"""

from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from zope import interface

from collective.templateengines.interfaces import *

from plone.app.portlets.portlets.rss import RSSFeed

__author__ = """Mikko Ohtamaa <mikko.ohtamaa@twinapex.com>"""
__docformat__ = 'epytext'
__copyright__ = "2009 Twinapex Research"
__license__ = "GPL"


class RSSFeedTag:

    interface.implements(ITag)
    
    # Hold RSSFeed objects using url as a key
    _rss_cache = {}
    
    
    def getName(self):
        return "rss_feed"
    
    def render(self, scriptingContext, url, cache_timeout=60):
        """
        TODO: Support portal local date format
        """
        
        try:
            feed = RSSTag._rss_cache[url]
        except:
            feed = RSSFeed(url, cache_timeout)
            RSSFeedTag._rss_cache[url] = feed
            
        feed.update()
        
        items = feed.items
        for i in items:
            # Patch in few helpful variables
            # Format blog entry timestamps for the template
            if "updated" in i:
                # all feeds don't provide this information
                i["friendly_date"] = i["updated"].strftime("%Y-%m-%d")
            
        return items
        
        
        
        
        
        
        
            
        
            
        
    
    
        


"""

    Tags for outputting content data.

"""

from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.Five.browser import BrowserView

from zope import interface

from collective.templateengines.interfaces import *
from Products.ZCatalog.CatalogBrains import AbstractCatalogBrain

__author__ = """Mikko Ohtamaa <mikko@redinnovation.com>"""
__docformat__ = 'epytext'
__copyright__ = "2008 Red Innovation Ltd."
__license__ = "GPL"

class ListFolder:

    interface.implements(ITag)
    
    def getName(self):
        return "list_folder"
    
    def render(self, context, folder, title=None, exclude_self=True, extra_items="", filters={}):
        """ List folder contents as bulleted link list. 
        
        @param context: ITemplateContext object
        
        @param folder: slashed path to the folder on the site, starting from the site root. No initial slash.
        @param title: optional <h3> level subheading which is shown if the folder contains at least one item
        @param filter: optional content type mask. only this type of content items is shown.
        @param exclude_self: Do not list the current context
        @param extra_items: Comma separated list of item absolute paths which lie outside the target folder, but should be included in the listing
        """
        
        # Get zope traversing context - don't confuse these two!
        context = context.getTraversingContext()
        
        # you know some object which is refered as "context"
        portal_url = getToolByName(context, "portal_url")
        portal_catalog = getToolByName(context, "portal_catalog")
        portal = portal_url.getPortalObject()
        mtool = getToolByName(context, "portal_membership") 
        
        def sanify_path(path):
            if path.startswith("/"):
                path = path[1:]
            return portal_url.getPortalPath() + "/" + path
            
        path = sanify_path(folder) 
        # Get access to the folder object using slash notation
        #folder=portal.restrictedTraverse()
                    
        search_dict = {
            "path" : { "query" : path, "depth" : 1 }
        }
        
        search_dict.update(filters)
            
        # Editors should be able to list items which are not yet publicly visible
        show_inactive = mtool.checkPermission('Access inactive portal content', context)        
        items = portal_catalog.queryCatalog(search_dict, show_inactive=show_inactive)
        
        if extra_items != "":
            extra_items = extra_items.split(" ,")
            
            items = list(items)
        
            for extra in extra_items:
                
                if extra == "":
                    # Skip stale etnries
                    continue
                
                # And then they wanted exception to the rule...
                path = sanify_path(extra)
                query = {
                         "path" : { "query" : path, "depth" : 0 }
                }     
                extra = portal_catalog.queryCatalog(query)
                items += list(extra)
        
        # Generate HTML snippets directly into a Python string
        output = ""
            
        if len(items) > 0:
            if title:
                output += "<h3>" + title + "</h3>"
                 
        output += """<ul class="folder-list">"""
        
        for item in items:
            if exclude_self and item.getURL() == context.absolute_url():
                continue
            output += """<li><a href="%s">%s</a></li>""" % (item.getURL(), item["Title"])
            
        output += """</ul>"""
        return output
    
    def __call__(self, *args, **kwargs):
        raise RuntimeError("Deprecated - TagProxy should call render() method")
    
    
class QueryTag(object):
    """ Perform portal_catalog search query and return results.
        
    """
    
    interface.implements(ITag)
    
    def getName(self):
        return "query"
    
    def render(self, scriptingContext, searchParameters):
        """     
        
        @param searchParameters: Dictionary of portal_catalog query parameters
        @param scriptingContext: Instance of collective.templateengines.interfaces.ITemplateContext
        @return: Raw catalog brains objects
        """
        
        mappings = scriptingContext.getMappings()
        
        # Get traversing context
        context = mappings["context"]
        request = mappings["request"]    
        
        portal_catalog = getToolByName(context, "portal_catalog")
        return portal_catalog.queryCatalog(searchParameters)
    
    
class ExploreTag(object):
    """ Dump object developer info.
        
    """
    
    interface.implements(ITag)
    
    def getName(self):
        return "explore"
    
    def renderObject(self, object, pp, buffer):
        print >> buffer, '<table class="object-explore">'
        print >> buffer, "<thead>"
        print >> buffer, "<tr>"
        print >> buffer, "<th>Attribute name</th>"        
        print >> buffer, "<th>Value</th>"        
        print >> buffer, "</tr>"        
        
        def do_obj(key, value):
            print >> buffer, "<tr>"
            print >> buffer, "<td>"            
            print >> buffer, str(key)
            print >> buffer, "</td>"           
            
            print >> buffer, "<td>"            
            pp.pprint(value)
            print >> buffer, "</td>"
            print >> buffer, "</tr>"            
            
        # Outputters, most specialized firsh
        if isinstance(object, AbstractCatalogBrain):
            # Zope catalog brain objects need special keying
            for key in object.__record_schema__.keys():
                value = object[key]
                do_obj(key, value)
        elif hasattr(object, "__dict__"):        
            for key, value in object.__dict__.items():
                do_obj(key, value)
        elif type(object) == type({}):
            for key, value in object.items():
                do_obj(key, value)            
        elif type(object) == type([]) or type(object) == type(()):
            # List & sequence
            for i in range(0, len(object)):
                self.do_obj(i, object[i])                            
        else:
            do_obj("Object", object)

        print >> buffer, "</tbody>"
        print >> buffer, "</table>"        
    
    def render(self, scriptingContext, object):
        """             
        Use Python PrettyPrinter to dump objects 
        to HTML table.
        """

        import pprint
        import StringIO
        buffer = StringIO.StringIO()
        pp = pprint.PrettyPrinter(indent=4, depth=3, stream=buffer)
        self.renderObject(object, pp, buffer)        
        return buffer.getvalue()
    
    
class LatestNewsView(BrowserView):
    
    index = ViewPageTemplateFile("latest_news.pt")
    
    def __call__(self):
        self.news_items = self.context.portal_catalog.queryCatalog({"portal_type":"News Item","sort_on":"Date","sort_order":"reverse","sort_limit":self.count,"review_state":"published"})
        return self.index()

class LatestNewsTag(object):
    """ Render latest news items """
    
    interface.implements(ITag)
    
    def getName(self):
        return "latest_news"

    def render(self, scriptingContext, count=3):
                
        mappings = scriptingContext.getMappings()
        
        # Get traversing context
        context = mappings["context"]
        request = mappings["request"]
                
        view = LatestNewsView(context, request)
        view.count = count
        # Put view into context acquistion chain
        view = view.__of__(context)
        
        return view()


from Products.Five import zcml
from Products.Five import fiveconfigure

from Testing import ZopeTestCase as ztc

from Products.PloneTestCase import PloneTestCase as ptc
from Products.PloneTestCase.layer import onsetup

from Products.statusmessages.interfaces import IStatusMessage

from collective.easytemplate import tagconfig, engine
from collective.templateengines.backends import jinja

@onsetup
def setup_app():

    fiveconfigure.debug_mode = True
    import collective.easytemplate
    zcml.load_config('configure.zcml', collective.easytemplate)
    fiveconfigure.debug_mode = False
    
    # We need to tell the testing framework that these products
    # should be available. This can't happen until after we have loaded
    # the ZCML.
    
    ztc.installPackage('collective.easytemplate')
    ztc.installProduct('Products.LinguaPlone')    
    ztc.installProduct('PlacelessTranslationService')
    

setup_app()
ptc.setupPloneSite(products=['collective.easytemplate', "Products.LinguaPlone:LinguaPlone"])

class EasyTemplateTestCase(ptc.PloneTestCase):
    """We use this base class for all the tests in this package. If necessary,
    we can put common utility or setup code in here.
    """

    def afterSetUp(self):
        # Might be another engine since some tests switch
        # this global variable
        engine.setDefaultEngine()

    def createContent(self):
        """ Create site content which is used in test cases. """
        self.loginAsPortalOwner()
        
        # Published folder
        self.portal.invokeFactory("Folder", "folder")
                
        # Published doc
        self.portal.folder.invokeFactory("Document", "doc")
        self.portal.folder.invokeFactory("Document", "doc_published")
        
        self.portal.folder.invokeFactory("TemplatedDocument", "easy_template")
        
        self.portal.folder.easy_template.setCatchErrors(True)
        
        self.portal.folder.doc_published.setTitle("Published Title")
        self.portal.folder.doc.setTitle("Unpublished")                
                
        self.portal.portal_workflow.doActionFor(self.portal.folder.doc_published, "publish")
        self.portal.portal_workflow.doActionFor(self.portal.folder.easy_template, "publish")
        self.portal.portal_workflow.doActionFor(self.portal.folder, "publish")

        # Update portal catalog
        self.portal.folder.reindexObject()
        self.portal.folder.easy_template.reindexObject()        
        self.portal.folder.doc.reindexObject()
        self.portal.folder.doc_published.reindexObject()
        
        self.logout()
    
    def runSnippet(self, str, admin=False, assumeErrors=False):
        """ Execute Jinja snippet in a secure context. 
        
        @param assumeErrors: There should be error output
        """
        doc = self.portal.folder.easy_template
        self.loginAsPortalOwner()
        doc.setText(str)
        
        if not admin:        
            self.logout()
            
        output = doc.getTemplatedText()
        messages = IStatusMessage(self.portal.REQUEST).showStatusMessages()        
        
        if messages:            
            for m in messages: 
                print "Template error:" + m.message
            
        if assumeErrors:
            # There should be no error messages
            self.assertNotEqual(len(messages), 0)
        else:
            self.assertEqual(len(messages), 0)
            
        return output    


"""

    Tags for outputting content data.

"""

from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from zope import interface

from collective.templateengines.interfaces import *

from Products.LinguaPlone.interfaces import ITranslatable
from Products.LinguaPlone.browser.selector import TranslatableLanguageSelector

__author__ = """Mikko Ohtamaa <mikko@redinnovation.com>"""
__docformat__ = 'epytext'
__copyright__ = "2008 Red Innovation Ltd."
__license__ = "GPL"

def getCurrentLanguage(context):
    # See plone.app.i18n.locales.browser.selector
    tool =  getToolByName(context, 'portal_languages', None)
    
    bound = tool.getLanguageBindings()
    current = bound[0]        
    return current

class CurrentLanguageTag(object):
    """ Return current active language. """

    interface.implements(ITag)
    
    def getName(self):
        return "current_language"
    
    def render(self, scriptingContext):
        
        # Get traversing context
        mappings = scriptingContext.getMappings()
        context = mappings["context"]
        request = mappings["request"]
        
        return getCurrentLanguage(context)
        
        
        
class TranslateTag(object):
    """ Look up values from translation dictionary.
        
    """
    
    interface.implements(ITag)
    
    def getName(self):
        return "translate"
    
    
    def render(self, scriptingContext, message, domain="plone", language=None, default=None):
        """             
        
        """

        # Get traversing context
        mappings = scriptingContext.getMappings()
        context = mappings["context"]
        request = mappings["request"]
        
        if language == None:
            language = getCurrentLanguage(context)
        
        tool = getToolByName(context, 'translation_service')        

        value = tool.utranslate(context=context, domain=domain, msgid=message, target_language=language, default=default)
        return value
        
        


import unittest

from Products.CMFCore.utils import getToolByName

from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.SecurityManagement import noSecurityManager
from AccessControl.User import User

from Products.statusmessages.interfaces import IStatusMessage

from collective.easytemplate.content.TemplatedDocument import ERROR_MESSAGE
from collective.easytemplate.tests.base import EasyTemplateTestCase

from collective.easytemplate import tagconfig, engine
from collective.templateengines.backends import cheetah
from collective.templateengines.utils import dump_messages

class ContentTestCase(EasyTemplateTestCase):
    """ Check that Templated Document functions properly. """
    
    def afterSetUp(self):
        
        EasyTemplateTestCase.afterSetUp(self)
        
        self.workflow = getToolByName(self.portal, 'portal_workflow')
        self.acl_users = getToolByName(self.portal, 'acl_users')
        self.types = getToolByName(self.portal, 'portal_types')
        
        # old tests - were written  for cheetah
        engine.setupEngine(cheetah.Engine())
        
        
    
    def test_create(self):        
        self.loginAsPortalOwner()
        self.portal.invokeFactory("TemplatedDocument", "doc")
        doc = self.portal.doc
        
    def test_render(self):
        self.loginAsPortalOwner()
        self.portal.invokeFactory("TemplatedDocument", "doc")
        doc = self.portal.doc
        tests = "Oh joy"
        doc.setCatchErrors(True)
        doc.setTitle(tests)
        doc.setText("${title}")
        
        output = doc.getTemplatedText()
        messages = IStatusMessage(self.portal.REQUEST).showStatusMessages()        
        
        if messages:
            for m in messages: print str(m.message)
            
        self.assertEqual(len(messages), 0) # No template error messages
        self.assertEqual(str(output), "<p>" + tests + "</p>") # At has automatic HTML wrap

        
    def test_render_failed(self):
        
        self.loginAsPortalOwner()
        self.portal.invokeFactory("TemplatedDocument", "doc")
        doc = self.portal.doc
        doc.setCatchErrors(True)
        doc.setText("#if")
        output = doc.getTemplatedText()
        
        
        messages = IStatusMessage(self.portal.REQUEST).showStatusMessages()        
        self.assertEqual(len(messages), 1) # No template error messages
        self.assertEqual(output, ERROR_MESSAGE)
        
    def test_list_folder(self):
        """ Test our custom template tag, list folder. """
        
        self.loginAsPortalOwner()
        self.portal.invokeFactory("Folder", "folder")
        self.portal.folder.invokeFactory("TemplatedDocument", "doc")
        self.portal.folder.invokeFactory("Document", "doc2")
        doc = self.portal.folder.doc
        doc.setCatchErrors(True)

        doc.setText('$list_folder($folder="folder")')
        #print doc.getRawText()                
        output = doc.getTemplatedText()
                
        # Should not happen
        messages = IStatusMessage(self.portal.REQUEST).showStatusMessages()                
        if messages:
            for m in messages: print "Template errors:" + str(m.message)
        
        
    
        self.assertEqual(len(messages), 0)
        
        # See that object ids appear in the output (in links)
        output = str(output)
        print output
        self.assertTrue("doc" in output)
        self.assertTrue("doc2" in output)
        
    def test_list_folder_extra_items(self):
        """ Test that directly pointing to items in multiple folders work. """
 
        self.loginAsPortalOwner()
        self.portal.invokeFactory("Folder", "folder")
        self.portal.folder.invokeFactory("TemplatedDocument", "doc")
        self.portal.folder.invokeFactory("Document", "doc2")
        
        self.portal.invokeFactory("Folder", "folder2")
        self.portal.folder2.invokeFactory("Folder", "not-listed-item")
        
        doc = self.portal.folder.doc
        doc.setCatchErrors(True)       

        doc.setText('$list_folder($folder="folder", $extra_items="folder2")')
        print doc.getRawText()                
        output = doc.getTemplatedText()
                
        # Should not happen
        messages = IStatusMessage(self.portal.REQUEST).showStatusMessages()                
        for m in messages: print str(m.message)
        
        self.assertEqual(len(messages), 0)
        
        # See that object ids appear in the output (in links)
        output = str(output)
        print output
        self.assertTrue("doc" in output)
        self.assertTrue("doc2" in output)
        self.assertTrue("folder2" in output)
        self.assertFalse("not-listed-item" in output)
        
    def test_unfiltered(self):
        """ Test unfiltered template input. 
        
        See that unfiltered code is succesfully evaluated if present.
        """
        
        # this is a test case for django
        engine.setDefaultEngine()        
        self.createContent()
        self.portal.folder.easy_template.setText("Foobar")
        self.portal.folder.easy_template.setUnfilteredTemplate("{{ explore(context) }}")
        output = self.portal.folder.easy_template.getTemplatedText()
        
        messages = IStatusMessage(self.portal.REQUEST).showStatusMessages()                
        for m in messages: print str(m.message)
        
        self.assertEqual(len(messages), 0)
                
        # Should have variables dumped         
        self.assertTrue("easy_template" in output)
            
def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(ContentTestCase))
    return suite



"""

    Maintain list of custom template tags.
    
    Just import & add new tags to this dictionary.
    
"""

from collective.easytemplate.tags.content import ListFolder, QueryTag, ExploreTag, LatestNewsTag
from collective.easytemplate.tags.views import ViewletTag, PortletTag, ViewTag
from collective.easytemplate.tags.rss import RSSFeedTag

# Registered tags - function name - ITag interface instance
tags = [ListFolder(), QueryTag(), ViewletTag(), ViewTag(), PortletTag(), RSSFeedTag(), ExploreTag(), LatestNewsTag() ]

try:
    # Add translations tags only if LinguaPlone is installed
    import Products.LinguaPlone
    from collective.easytemplate.tags.i18n import CurrentLanguageTag, TranslateTag
    tags.append(CurrentLanguageTag())
    tags.append(TranslateTag())
except:
    pass


"""

    Kupu editable HTML portlet which takes in template tags.
    
    http://www.redinnovation.com
    
"""
__author__ = """Mikko Ohtamaa <mikko@redinnovation.com>"""
__docformat__ = 'epytext'
__copyright__ = "2008 Red Innovation Ltd."
__license__ = "GPL"


import logging

from zope.interface import implements
from zope.component import getUtility
from zope import schema
from zope.formlib import form


from plone.app.portlets.portlets import base

from plone.portlet.static import static 
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from plone.app.form.widgets.wysiwygwidget import WYSIWYGWidget

from interfaces import ITemplatedPortlet
from collective.easytemplate.utils import applyTemplate
from collective.easytemplate.engine import getEngine, getTemplateContext

# TODO: Fix to use own message factory
from plone.portlet.static.static import _

logger = logging.getLogger("Plone")

class TemplatedPortletRenderer(static.Renderer):
    """ Overrides static.pt in the rendering of the portlet. """
    
    def getTemplateContext(self):
        return getTemplateContext(self.context)
    
    def getHeader(self):
        """ Get cooked portlet title.
        """
        context = self.getTemplateContext()
        header = self.data.header
        header, errors = applyTemplate(context, header, logger)
        
        if errors:
            return "Errors"
        else:
            return header
        
    def getBody(self):
        """ Get cooked portlet body. """
        context = self.getTemplateContext()
        text = self.data.text
        text, errors = applyTemplate(context, text, logger)
        
        if errors:
            return "Errors"
        else:
            logger.debug("Got:" + text)
            return text
            
    render = ViewPageTemplateFile('templated_portlet.pt')
    
class TemplatedPortletAssignment(static.Assignment):
    """ Assigner for templated portlet. """
    implements(ITemplatedPortlet)
    
class TemplatedPortletAddForm(static.AddForm):
    """ Make sure that add form creates instances of our custom portlet instead of the base class portlet. """
    
    label = _(u"title_add_templated_portlet",
              default=u"Add templated text portlet")
    
    description = _(u"description_static_portlet",
                    default=u"A portlet which can display HTML text with dynamic tags")
    
    
    def create(self, data):
        return TemplatedPortletAssignment(**data)    
    
class TemplatedPortletEditForm(static.EditForm):
    """ Edit view for templated portlet. """
    form_fields = form.Fields(ITemplatedPortlet)
    form_fields['text'].custom_widget = WYSIWYGWidget
    label = _(u"title_edit_static_portlet",
              default=u"Edit templated text portlet")
    description = _(u"description_static_portlet",
                    default=u"A portlet which can display HTML text with dynamic tags.")

    
    


# -*- coding: utf-8 -*-

from zope.interface import Interface

##code-section HEAD
##/code-section HEAD

class ITemplatedDocument(Interface):
    """Marker interface for .TemplatedDocument.TemplatedDocument
    """
    
class ITEmplateContext(Interface):
    """ Hold information about variables exposed to the template language. """
    
    def getAsMapping():
        """ Return exposed variables as a dictionary. """
    
class ITemplateLanguageEngine(Interface):
    
    def makeTemplateFromString(s):
        """
        @param s: Template source code as string
        @return (ITemplate, sequence of ITEmplateMessages)
        """
        
class ITemplate(Interface):
    """ Holds one template.
    
    Template can be string/file sourced and processed.
    """
    
    def execute(context):
        """ 
        @param context: ITemplateContext object
        @return: (evaluation result as string, sequence of ITEmplateMessages)
        """

##code-section FOOT
##/code-section FOOT


# -*- coding: utf-8 -*-
"""
This module contains the tool of collective.easytemplate
"""
import os
from setuptools import setup, find_packages

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

version = '0.7'


long_description = read('docs', 'Introduction.txt') + \
    read('docs', 'TemplateEngine.txt') + \
    read('docs', 'Objects.txt') + \
    read('docs', 'Security.txt') + \
    read('docs', 'Authoring.txt') + \
    read('docs', 'Context.txt') + \
    read('docs', 'Tags.txt') + \
    read('docs', 'RegisteringTags.txt') 
    
tests_require=['zope.testing']

setup(name='collective.easytemplate',
      version=version,
      description="Simple scripting for Plone pages",
      long_description=long_description,
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        'Framework :: Plone',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        ],
      keywords='plone cheetah jinja template kupu scripting dynamic page',
      author='Mikko Ohtamaa',
      author_email='mikko.ohtamaa@twinapex.com',
      url='http://plone.org/products/easy-template',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['collective', ],
      include_package_data=True,
      zip_safe=False,
      install_requires=['setuptools',
                        'Jinja2',
                        'plone.portlet.static'
                        # -*- Extra requirements: -*-
                        ],
      tests_require=tests_require,
      extras_require=dict(tests=tests_require),
      test_suite = 'collective.easytemplate.tests',
      entry_points="""
      # -*- entry_points -*- 
      [distutils.setup_keywords]
      paster_plugins = setuptools.dist:assert_string_list

      [egg_info.writers]
      paster_plugins.txt = setuptools.command.egg_info:write_arg
      """,
      paster_plugins = ["ZopeSkel"],
      )



"""

    Test Zope security wrapped for Jinja 2

"""

__author__ = """Mikko Ohtamaa <mikko@redinnovation.com>"""
__docformat__ = 'plaintext'
__copyright__ = "2008 Red Innovation Ltd."
__license__ = "GPL"

from Products.CMFCore.utils import getToolByName

from Products.statusmessages.interfaces import IStatusMessage

from collective.easytemplate.content.TemplatedDocument import ERROR_MESSAGE
from collective.easytemplate.tests.base import EasyTemplateTestCase

from zope.component import getUtility, getMultiAdapter, getSiteManager
from zope.component.interfaces import IObjectEvent
from zope.interface import implements

from collective.easytemplate import tagconfig, engine
from collective.templateengines.backends import jinja

class TestSecurity(EasyTemplateTestCase):
    
    def afterSetUp(self):        
        EasyTemplateTestCase.afterSetUp(self)
        self.createContent()
                    
    def test_00_assume_jinja(self):
        """ Only Jinja backend supports security.
        """
        self.assertTrue(isinstance(engine.getEngine(), jinja.Engine))

    def test_bad_attribute(self):
        """ Test attribute name which does not exist. """
        val = self.runSnippet("Test {{ context.foobar }}")
        print "Got val:" + str(val)
        raise RuntimeError("Undefined variables are not yet caught - remember to fix this please")
        
    def test_allowed_traverse(self):
        val = self.runSnippet("Test {{ context.aq_parent.doc_published.Title() }}")
        self.assertEqual(val, "<p>Test Published Title</p>")

    def test_unallowed_traverse(self):
        """ doc is unpublished object """
        
        from AccessControl.unauthorized import Unauthorized
        #unauthorized = AccessControl.Unauthorized
        
        try:
            val = self.runSnippet("Test {{ context.aq_parent.doc.Title() }}")
            self.assertTrue(False) # Shold not be reached
        except Unauthorized:
            pass
                        
    def test_allowed_attribute(self):
        val = self.runSnippet("Test {{ context.schema }}")

            
    def test_unallowed_attribute(self):
        
        from AccessControl.unauthorized import Unauthorized
                
        try:
            val = self.runSnippet("Test {{ context.__ac_roles__}}")
            self.assertTrue(False) # Shold not be reached
        except Unauthorized:
            pass
       
    def test_unallowed_method(self):
        """ Anonymous user should not be able to call setTitle """

        from AccessControl.unauthorized import Unauthorized
                
        try:
            val = self.runSnippet("Test {{ context.setTitle('abc') }}")
            self.assertTrue(False) # Shold not be reached
        except Unauthorized:
            pass
        
    def test_unallowed_traversed_method(self):
        """ Anonymous user should not be able to call setTitle on parent """

        from AccessControl.unauthorized import Unauthorized
                
        try:
            val = self.runSnippet("Test {{ context.aq_parent.setTitle('abc') }}")
            self.assertTrue(False) # Shold not be reached
        except Unauthorized:
            pass

    def test_access_private_as_admin(self):
        """ Template which prefers to unpublished content should be available as admin"""
        val = self.runSnippet("Test {{ context.aq_parent.doc.Title() }}", admin=True)        
        self.assertEqual(val, "<p>Test Unpublished</p>")
        
    def test_set_title_as_admin(self):
        """ Call a setter method when logged in as admin.
        
        This should be allowed, though not useful."""
        val = self.runSnippet("Test {{ context.aq_parent.setTitle('abc') }}", admin=True)                        
        self.assertEqual(val, "<p>Test None</p>")


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestSecurity))
    return suite
